extern crate proc_macro;

use proc_macro::TokenStream;
use quote::quote;
use syn::{parse_macro_input, DeriveInput};

#[proc_macro_derive(Id)]
pub fn id_derive(input: TokenStream) -> TokenStream {
	// Parse the input tokens into a syntax tree
	let ast = parse_macro_input!(input as DeriveInput);
	let name = &ast.ident;
	let ty = match &ast.data {
		syn::Data::Struct(s) => {
			if let syn::Fields::Unnamed(fields) = &s.fields {
				if fields.unnamed.len() != 1 {
					return syn::Error::new_spanned(&ast, "Expected exactly one field.").to_compile_error().into();
				}
				&fields.unnamed[0].ty
			} else {
				return syn::Error::new_spanned(&ast, "Expected a tuple struct.").to_compile_error().into();
			}
		}
		_ => {
			return syn::Error::new_spanned(&ast, "Expected a struct.").to_compile_error().into();
		}
	};

	// Generate the code for the implementation
	let expanded = quote! {
		impl #name {
			#[inline]
			pub const fn new(id: #ty) -> Self {
				Self(id)
			}

			#[inline]
			pub const fn get(self) -> #ty {
				self.0
			}
		}
	};

	// Convert the generated code into a token stream and return it
	TokenStream::from(expanded)
}
